@@ -26,10 +26,7 @@ formatColumns = function(table, columns, template, ...) {
 #' @param mark the marker after every \code{interval} decimals in the numbers
 #' @param dec.mark a character to indicate the decimal point
 #' @param before whether to place the currency symbol before or after the values
-#' @param method the method(s) to convert a date to string in JavaScript; see
-#'   \code{DT:::DateMethods} for a list of possible methods, and
-#'   \url{http://mzl.la/1xGe99W} for a full reference
-#' @references See \url{http://rstudio.github.io/DT/functions.html} for detailed
+#' @references See \url{https://rstudio.github.io/DT/functions.html} for detailed
 #'   documentation and examples.
 #' @export
 #' @examples # !formatR
@@ -74,26 +71,54 @@ formatString = function(table, columns, prefix = '', suffix = '') {
 #' @export
 #' @rdname formatCurrency
 #' @param digits the number of decimal places to round to
-formatPercentage = function(table, columns, digits = 0) {
-  formatColumns(table, columns, tplPercentage, digits)
+formatPercentage = function(
+  table, columns, digits = 0, interval = 3, mark = ',', dec.mark = getOption('OutDec')
+) {
+  formatColumns(table, columns, tplPercentage, digits, interval, mark, dec.mark)
 }
 
 #' @export
 #' @rdname formatCurrency
-formatRound = function(table, columns, digits = 2) {
-  formatColumns(table, columns, tplRound, digits)
+formatRound = function(
+  table, columns, digits = 2, interval = 3, mark = ',', dec.mark = getOption('OutDec')
+) {
+  formatColumns(table, columns, tplRound, digits, interval, mark, dec.mark)
 }
 
 #' @export
 #' @rdname formatCurrency
-formatSignif = function(table, columns, digits = 2) {
-  formatColumns(table, columns, tplSignif, digits)
+formatSignif = function(
+  table, columns, digits = 2, interval = 3, mark = ',', dec.mark = getOption('OutDec')
+) {
+  formatColumns(table, columns, tplSignif, digits, interval, mark, dec.mark)
 }
 
 #' @export
 #' @rdname formatCurrency
-formatDate = function(table, columns, method = 'toDateString') {
-  formatColumns(table, columns, tplDate, method)
+#' @param method the method(s) to convert a date to string in JavaScript; see
+#'   \code{DT:::DateMethods} for a list of possible methods, and
+#'   \url{http://mzl.la/1xGe99W} for a full reference
+#' @param params a list parameters for the specific date conversion method,
+#'   e.g., for the \code{toLocaleDateString()} method, your browser may support
+#'   \code{params = list('ko-KR', list(year = 'numeric', month = 'long', day =
+#'   'numeric'))}
+formatDate = function(table, columns, method = 'toDateString', params = NULL) {
+  x = table$x
+  if (x$filter != 'none') {
+    if (inherits(columns, 'formula')) columns = all.vars(columns)
+    colnames = base::attr(x, 'colnames', exact = TRUE)
+    rownames = base::attr(x, 'rownames', exact = TRUE)
+    if (is.null(params)) params = list()
+    cols = sprintf("%d", name2int(columns, colnames, rownames))
+    x$filterDateFmt = as.list(x$filterDateFmt)
+    for (col in cols) x$filterDateFmt[[col]] = list(
+      method = method, params = toJSON(params)
+    )
+    table$x = x
+  }
+  # the code above is used to ensure the date(time) filter displays the same format or
+  # timezone as the column value
+  formatColumns(table, columns, tplDate, method, params)
 }
 
 #' @param valueColumns indices of the columns from which the cell values are
@@ -148,36 +173,50 @@ appendFormatter = function(js, name, names, rownames = TRUE, template, ...) {
   }
   i = name2int(name, names, rownames)
   JS(append(
-    js, after = 1,
+    js, after = length(js) - 1,
     template(i, ..., names, rownames)
   ))
 }
 
 tplCurrency = function(cols, currency, interval, mark, digits, dec.mark, before, ...) {
   sprintf(
-    "DTWidget.formatCurrency(this, row, data, %d, '%s', %d, %d, '%s', '%s', %s);",
-    cols, currency, digits, interval, mark, dec.mark, if (before) 'true' else 'false'
+    "DTWidget.formatCurrency(this, row, data, %d, %s, %d, %d, %s, %s, %s);",
+    cols, jsValues(currency), digits, interval, jsValues(mark), jsValues(dec.mark),
+    jsValues(before)
   )
 }
 
 tplString = function(cols, prefix, suffix, ...) {
-  sprintf("DTWidget.formatString(this, row, data, %d, '%s', '%s');", cols, prefix, suffix)
+  sprintf(
+    "DTWidget.formatString(this, row, data, %d, %s, %s);",
+    cols, jsValues(prefix), jsValues(suffix)
+  )
 }
 
-tplPercentage = function(cols, digits, ...) {
-  sprintf("DTWidget.formatPercentage(this, row, data, %d, %s);", cols, digits)
+tplPercentage = function(cols, digits, interval, mark, dec.mark, ...) {
+  sprintf(
+    "DTWidget.formatPercentage(this, row, data, %d, %d, %d, %s, %s);",
+    cols, digits, interval, jsValues(mark), jsValues(dec.mark)
+  )
 }
 
-tplRound = function(cols, digits, ...) {
-  sprintf("DTWidget.formatRound(this, row, data, %d, %d);", cols, digits)
+tplRound = function(cols, digits, interval, mark, dec.mark, ...) {
+  sprintf(
+    "DTWidget.formatRound(this, row, data, %d, %d, %d, %s, %s);",
+    cols, digits, interval, jsValues(mark), jsValues(dec.mark)
+  )
 }
 
-tplSignif = function(cols, digits, ...) {
-  sprintf("DTWidget.formatSignif(this, row, data, %d, %d);", cols, digits)
+tplSignif = function(cols, digits, interval, mark, dec.mark, ...) {
+  sprintf(
+    "DTWidget.formatSignif(this, row, data, %d, %d, %d, %s, %s);",
+    cols, digits, interval, jsValues(mark), jsValues(dec.mark)
+  )
 }
 
-tplDate = function(cols, method, ...) {
-  sprintf("DTWidget.formatDate(this, row, data, %d, '%s')", cols, method);
+tplDate = function(cols, method, params, ...) {
+  params = if (length(params) > 0) paste(',', toJSON(params)) else ''
+  sprintf("DTWidget.formatDate(this, row, data, %d, %s%s);", cols, jsValues(method), params)
 }
 
 DateMethods = c(
@@ -200,17 +239,27 @@ tplStyle = function(cols, valueCols, target, styles, ...) {
   switch(
     target,
     cell = sprintf(
-      "var value=data[%s]; if (value!==null) $(this.api().cell(row, %s).node()).css({%s});",
+      "var value=data[%s]; $(this.api().cell(row, %s).node()).css({%s});",
       valueCols, cols, css
     ),
     row = sprintf(
-      "var value=data[%s]; if (value!==null) $(row).css({%s});",
+      "var value=data[%s]; $(row).css({%s});",
       valueCols, css
     ),
     stop('Invalid target!')
   )
 }
 
+jsValues = function(x) {
+  if (inherits(x, c("POSIXt", "POSIXct", "POSIXlt"))) {
+    x = format(x, "%Y-%m-%dT%H:%M:%SZ", tz = "UTC")
+  } else if (inherits(x, "Date")) {
+    x = format(x, "%Y-%m-%d")
+  }
+  vapply(x, jsonlite::toJSON, character(1), auto_unbox = TRUE)
+}
+
+
 #' Conditional CSS styles
 #'
 #' A few helper functions for the \code{\link{formatStyle}()} function to
@@ -242,10 +291,11 @@ styleInterval = function(cuts, values) {
   n = length(cuts)
   if (n != length(values) - 1)
     stop("length(cuts) must be equal to length(values) - 1")
-  values = sprintf("'%s'", values)
+  values = jsValues(values)
   if (n == 0) return(values)
   if (!all(cuts == sort(cuts))) stop("'cuts' must be sorted increasingly")
-  js = ''
+  js = "isNaN(parseFloat(value)) ? '' : "  # missing or non-numeric values in data
+  cuts = jsValues(cuts)
   for (i in seq_len(n)) {
     js = paste0(js, sprintf('value <= %s ? %s : ', cuts[i], values[i]))
   }
@@ -254,18 +304,23 @@ styleInterval = function(cuts, values) {
 
 #' @param levels a character vector of data values to be mapped (one-to-one) to
 #'   CSS values
+#' @param default a string used as the the default CSS value for values other than levels
 #' @export
 #' @rdname styleInterval
-styleEqual = function(levels, values) {
+styleEqual = function(levels, values, default = "") {
   n = length(levels)
   if (n != length(values))
     stop("length(levels) must be equal to length(values)")
+  if (!is.character(default) || length(default) != 1)
+    stop("default must be a string")
   if (n == 0) return("''")
+  levels = jsValues(levels)
+  values = jsValues(values)
   js = ''
   for (i in seq_len(n)) {
-    js = paste0(js, sprintf("value == '%s' ? '%s' : ", levels[i], values[i]))
+    js = paste0(js, sprintf("value == %s ? %s : ", levels[i], values[i]))
   }
-  JS(paste0(js, "''"))
+  JS(paste0(js, jsValues(default)))
 }
 
 #' @param data a numeric vector whose range will be used for scaling the
@@ -283,7 +338,7 @@ styleColorBar = function(data, color, angle=90) {
   rg = range(data, na.rm = TRUE, finite = TRUE)
   r1 = rg[1]; r2 = rg[2]; r = r2 - r1
   JS(sprintf(
-    "isNaN(parseFloat(value)) || value <= %s ? '' : 'linear-gradient(%sdeg, transparent ' + (%s - value)/%s * 100 + '%%, %s ' + (%s - value)/%s * 100 + '%%)'",
+    "isNaN(parseFloat(value)) || value <= %f ? '' : 'linear-gradient(%fdeg, transparent ' + (%f - value)/%f * 100 + '%%, %s ' + (%f - value)/%f * 100 + '%%)'",
     r1, angle, r2, r, color, r2, r
   ))
 }
